@@ -33,13 +33,18 @@ def make_aov_pass(
         f'Invalid style {style} for AOV Output Node, must be in {valid_styles}.'
     # Go through existing passes and make sure it doesn't exist before creating
     view_layer = zpy.blender.verify_view_layer()
-    if view_layer.get('aovs', None) is not None:
-        for aov in view_layer['aovs']:
+    if view_layer.cycles.get('aovs', None) is not None:
+        for aov in view_layer.cycles.aovs.values():
             if aov['name'] == style:
                 log.info(f'AOV pass for {style} already exists.')
                 return
-    bpy.ops.cycles.add_aov()
-    view_layer['aovs'][-1]['name'] = style
+    if int(bpy.app.version_string.split('.')[1]) >= 92:
+        # This call works in 2.92
+        bpy.ops.scene.view_layer_add_aov()
+    else:
+        # This call works in 2.91
+        bpy.ops.cycles.add_aov()
+    view_layer.cycles.aovs[-1]['name'] = style
     view_layer.update()
     log.info(f'Created AOV pass for {style}.')
 
@@ -163,10 +168,10 @@ def hsv_node(
 
 @gin.configurable
 def render_aov(
-    rgb_path: Union[str, Path] = None,
-    depth_path: Union[str, Path] = None,
-    iseg_path: Union[str, Path] = None,
-    cseg_path: Union[str, Path] = None,
+    rgb_path: Union[Path, str] = None,
+    depth_path: Union[Path, str] = None,
+    iseg_path: Union[Path, str] = None,
+    cseg_path: Union[Path, str] = None,
     width: int = 640,
     height: int = 480,
     hsv: Tuple[float] = None,
@@ -185,6 +190,7 @@ def render_aov(
     scene.frame_start = scene.frame_current
     scene.render.use_file_extension = False
     scene.render.use_stamp_frame = False
+    scene.render.filepath = ''
 
     # Create AOV output nodes
     render_outputs = {
@@ -235,7 +241,7 @@ def render_aov(
         _mute_aov_file_output_node('instance', mute=True)
         _mute_aov_file_output_node('depth', mute=True)
         _mute_aov_file_output_node('rgb', mute=False)
-        _rgb_render_settings()
+        default_render_settings()
         _render()
 
     cseg_is_on = (render_outputs.get('category', None) is not None)
@@ -247,7 +253,7 @@ def render_aov(
         _mute_aov_file_output_node('instance', mute=(not iseg_is_on))
         _mute_aov_file_output_node('depth', mute=(not depth_is_on))
         _mute_aov_file_output_node('rgb', mute=True)
-        _seg_render_settings()
+        segmentation_render_settings()
         _render()
 
     # Save intermediate scene
@@ -277,7 +283,7 @@ def _mute_aov_file_output_node(style: str, mute: bool = True):
         _output_node.mute = mute
 
 
-def _rgb_render_settings():
+def default_render_settings():
     """ Render settings for normal color images. """
     scene = zpy.blender.verify_blender_scene()
     scene.render.engine = "CYCLES"
@@ -311,7 +317,7 @@ def _rgb_render_settings():
     scene.display.shading.show_specular_highlight = True
 
 
-def _seg_render_settings():
+def segmentation_render_settings():
     """ Render settings for segmentation images. """
     scene = zpy.blender.verify_blender_scene()
     scene.render.engine = "CYCLES"
@@ -345,12 +351,19 @@ def _seg_render_settings():
     scene.display.shading.show_specular_highlight = False
 
 
-def _render(threads: int = 4,
-            logfile: str = 'blender_render.log',
-            ):
-    """ Render in Blender. """
+def _render(
+    threads: int = 4,
+    logfile_path: Union[Path, str] = 'blender_render.log',
+) -> None:
+    """ The actual call to render a frame in Blender.
+
+    Args:
+        threads (int, optional): Number of threads to render on. Defaults to 4.
+        logfile_path (Union[Path, str]): Path to save render logfile.
+    """
     start_time = time.time()
     scene = zpy.blender.verify_blender_scene()
+    # TODO: Get a better default number based on number of available cores
     scene.render.threads = threads
     # TODO: The commented out code here only works on Linux (fails on Windows)
     # try:
@@ -365,7 +378,7 @@ def _render(threads: int = 4,
     #     log.warning(f'Render log removal raised exception {e}')
     try:
         # This is the actual render call
-        bpy.ops.render.render(write_still=True)
+        bpy.ops.render.render()
     except Exception as e:
         log.warning(f'Render raised exception {e}')
     # try:
